﻿using AAD_KeyVault_Certificate_Authentication.Abstractions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.Graph;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AAD_KeyVault_Certificate_Authentication.Controllers
{
    [ApiController]
    [Route("api/aad/")]
    public class AADController : ControllerBase
    {
        private readonly ILogger<AADController> _logger;
        private readonly IGraphClient graphClient;

        public AADController(
            ILogger<AADController> logger,
            IGraphClient graphClient)
        {
            _logger = logger;
            this.graphClient = graphClient;
        }

        [HttpGet]
        [Route("getusers")]
        public async Task<IActionResult> GetUsers()
        {
            List<dynamic> users = new List<dynamic>();

            await foreach (User user in this.graphClient.EnumerateUsersAsync())
            {
                users.Add(new
                {
                    Id = user.Id,
                    Name = user.DisplayName
                });
            }

            return Ok(users);
        }

        [HttpGet]
        [Route("getusergroups")]
        public async Task<IActionResult> GetUserGroups(Guid userId)
        {
            List<dynamic> groups = new List<dynamic>();

            var user = await graphClient.GetUserByIdAsync(userId);
            
            List<Group> userGroups = user
              .MemberOf?
              .OfType<Group>()
              .ToList() ?? new List<Group>();

            groups.AddRange(userGroups.Select(x => new
            {
                Name = x.DisplayName,
                Id = x.Id
            }));

            return Ok(groups);
        }
    }
}
