﻿using AAD_KeyVault_Certificate_Authentication.Abstractions;
using Microsoft.Graph;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AAD_KeyVault_Certificate_Authentications.EntityEnumerators
{
    /// <summary>
    /// A provider for group enumeration which uses <see cref="GraphServiceClient"/> to get groups and 
    /// is passed to the <see cref="BatchAsyncEnumerable<>"/> to enumerate and set pages behind the scene
    /// </summary>
    public class BatchGroupsGraphProvider : IBatchGraphProvider<Group>
    {
        private readonly GraphServiceClient graphServiceClient;
        private IGraphServiceGroupsCollectionPage groupsPage;

        public BatchGroupsGraphProvider(GraphServiceClient graphServiceClient)
        {
            this.graphServiceClient = graphServiceClient;
        }

        public List<Group> GetCurrentPage()
        {
            return groupsPage.CurrentPage.ToList();
        }

        public async Task InitializeAsync()
        {
            groupsPage = await
              graphServiceClient.Groups
              .Request()
              .Select("id, displayName")
              .GetAsync();
        }

        public async Task<bool> NextPageAsync()
        {
            if (groupsPage.NextPageRequest is null)
            {
                return false;
            }
            else
            {
                groupsPage = await groupsPage.NextPageRequest.GetAsync();
                return true;
            }
        }
    }
}
