﻿using AAD_KeyVault_Certificate_Authentication.Abstractions;
using Microsoft.Graph;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AAD_KeyVault_Certificate_Authentications.EntityEnumerators
{
    /// <summary>
    /// A provider for user enumeration which uses <see cref="GraphServiceClient"/> to get groups and 
    /// is passed to the <see cref="BatchAsyncEnumerable<>"/> to enumerate and set pages behind the scene
    /// </summary>
    public class BatchUsersGraphProvider : IBatchGraphProvider<User>
    {
        private readonly GraphServiceClient graphServiceClient;
        private IGraphServiceUsersCollectionPage usersPage;

        public BatchUsersGraphProvider(GraphServiceClient graphServiceClient)
        {
            this.graphServiceClient = graphServiceClient;
        }

        public List<User> GetCurrentPage()
        {
            return usersPage.CurrentPage.ToList();
        }

        public async Task InitializeAsync()
        {
            usersPage = await
                 graphServiceClient.Users
                 .Request()
                 .Expand("MemberOf")
                 .GetAsync();
        }

        public async Task<bool> NextPageAsync()
        {
            if (usersPage.NextPageRequest is null)
            {
                return false;
            }
            else
            {
                usersPage = await usersPage.NextPageRequest.GetAsync();
                return true;
            }
        }
    }
}
