﻿using System.Collections.Generic;
using System.Threading.Tasks;

namespace AAD_KeyVault_Certificate_Authentication.Abstractions
{
    public class BatchAsyncEnumerator<TEntity> : IAsyncEnumerator<TEntity>
    {
        private readonly IBatchGraphProvider<TEntity> graphProvider;
        private bool initialized = false;
        private List<TEntity> currentPage = new List<TEntity>();

        /// <summary>
        /// The index of current property in current page
        /// </summary>
        public int currentIndexInPage = -1;

        public TEntity Current => currentPage[currentIndexInPage];

        public BatchAsyncEnumerator(IBatchGraphProvider<TEntity> graphProvider)
        {
            this.graphProvider = graphProvider;
        }

        public ValueTask DisposeAsync()
        {
            //
            return ValueTask.CompletedTask;
        }

        public async ValueTask<bool> MoveNextAsync()
        {
            if (!initialized)
            {
                initialized = true;
                await this.graphProvider.InitializeAsync();
                this.currentPage = this.graphProvider.GetCurrentPage();
                if (this.currentPage.Count == 0)
                {
                    return false;
                }
                this.currentIndexInPage = 0;
                return true;
            }

            if (currentIndexInPage < currentPage.Count - 1)
            {
                currentIndexInPage++;
                return true;
            }

            bool hasNextPage = await graphProvider.NextPageAsync();
            if (hasNextPage)
            {
                this.currentPage = graphProvider.GetCurrentPage();
                if (this.currentPage.Count == 0)
                {
                    return false;
                }
                this.currentIndexInPage = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
